<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Factory\CpAssinaturaDigitalFactory as AssinaturaFactory;
use Util\SMTP as SMTPService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\ConnectionCustom;
use Util\NewCode;

require_once ('../vendor/pear/http_request2/HTTP/Request2.php');

class CpAssinaturaDigitalService {

    protected $entityManager;
    protected $connection;
    protected $environment;
    /** @var AssinaturaFactory */
    protected $assinaturaFactory;
    protected $urlIntegracaoCertisign;
    protected $smtpService;
    protected $parametrizacoesServicesFactory;
    protected $connectionCustom;
    protected $newCode;
    

    public function __construct(EntityManager $entityManager, $environment, $urlIntegracaoCertisign, ConnectionCustom $connectionCustom) {
        $this->entityManager          = $entityManager;
        $this->environment            = $environment;
        $this->urlIntegracaoCertisign = $urlIntegracaoCertisign;
        $this->connectionCustom       = $connectionCustom;
        
        $this->iniciarDependencias(null);
    }
    
    private function iniciarDependencias($nrorg) {
        if (is_null($nrorg))
            $nrorg = $this->environment->getNrOrg();
            
        $this->setConnection($nrorg);
        $this->assinaturaFactory = new AssinaturaFactory($this->entityManager);
        $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        }
        else {
            $this->connection = $this->entityManager->getConnection();
        }
    }

    public function convertPDFinBase64($documento){
        try {

           // var_dump($documento); die;
            $handle = fopen($documento,'r');
            $file_content = fread($handle,filesize($documento));
            fclose($handle);

            // $byte_array = unpack("C*", $file_content);

            $encoded = base64_encode($file_content);

            return $encoded;

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function buildJsontoUpload($nmDocumento, $documento){
        
        $base64 = $this->convertPDFinBase64($documento);

        $arrayupload = array(
            "fileName" => $nmDocumento,
            "bytes"  => $base64
        );

        $jsonUpload = json_encode($arrayupload, JSON_PRETTY_PRINT);

        return $jsonUpload;
    }

    public function uploadtoCertsing($nmDocumento, $documento){
        
        $token = $this->getTokenCertsign();
        
        $return = array();
        
        $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/upload', \Http_Request2::METHOD_POST);
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'Content-Type' => 'application/json',
            'Token' => $token,
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(\Http_Request2::METHOD_POST);

        $body = $this->buildJsontoUpload($nmDocumento, $documento);

        // Request body
        $request->setBody($body);
        
        $return['request']['parameters'] = $parameters;
        $return['request']['body'] = json_decode($body, true);

        try {
            $response = $request->send();
            
            $retorno = $response->getBody();
            $retorno = json_decode($retorno, true);
            
            $return['response'] = $retorno;
            return $return;
        }
        catch (\HTTP_Request2_Exception $ex) {
            //echo $ex;
            $return['response'] = $ex;
            return $return;
        }
    }

    public function buildJsontoCreate($uploadId, $nmDocumento, $signers, $eletrSigners, $emailOperResp, $nrorg){
        
        $tipoDoc = null;
        $padraoAssinatura =  null;

        /* -- Documento -- */ 
        $document = array(
            "name" => $nmDocumento,
            "upload" => array("id" => $uploadId,
                              "name" => $nmDocumento)
            );

        /* -- responsável pelo envio -- */

        $params = array(
            'NRORG' => $nrorg
        );

        $responsavel =  $this->assinaturaFactory->getDadosSender($params);
            
        $senderAux = array();
         
        foreach ($responsavel as $resp) {
            $senderAux['NOME']  = $resp['NMSENDERCERTSIGN'];
            $senderAux['EMAIL'] = $resp['DSEMAILSENDERCERTSIGN'];
            $senderAux['CPF']   = $resp['NRCPFSENDERCERTSIGN'];
        }

        $sender = array(
            "name"  => $senderAux['NOME'],
            "email" => $senderAux['EMAIL'],
            "individualIdentificationCode" => $senderAux['CPF']
        );
        
        /* -- Recebe o Tipo de Documento e email do operador responsável -- */ 
        
        $params = array(
            'NRORG' => $nrorg
        );

        $dadosDoc =  $this->assinaturaFactory->getDadosSenderRoutes($params);

        foreach($dadosDoc as $dados) {
            $tipoDoc = $dados['NRTIPODOCCERTSIGN'];
            $padraoAssinatura = $dados['IDPADRAOASSINACERTSIGN'];
        }

        if ($tipoDoc == null){
            $tipoDoc = (int)$responsavel[0]['NRTIPODOCCERTSIGN'];    
        };
 

        if ($padraoAssinatura == null){
            $padraoAssinatura = (int)$responsavel[0]['IDPADRAOASSINACERTSIGN'];    
        };
        
        //$tipoDoc = '2889';

        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }

        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "document" => $document,
            "sender"  => $sender,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );
        
        if($padraoAssinatura){
            $arrayCreate["signatureStandard"] = $padraoAssinatura;
        }
        if($tipoDoc){
            $arrayCreate["typeId"] = $tipoDoc;
        }
        if($emailOperResp){
            $arrayCreate["recipients"] = array(
                "emails" => $emailOperResp,
                "attached" => true
            );
        }
        
        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);
        
        return $jsonCreate;
    }

    public function createSignatureCertsing($uploadId, $nmDocumento, $signers, $eletrSigners, $emailOperResp, $nrorg){
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/create', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
    
            $url->setQueryVariables($parameters);
    
            $request->setMethod(\Http_Request2::METHOD_POST);
    
            $body = $this->buildJsontoCreate($uploadId, $nmDocumento, $signers, $eletrSigners, $emailOperResp, $nrorg);
            $body = str_replace('\/', '/', $body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
            // Request body
            $request->setBody($body);
             
            try {
                $response = $request->send();
                
                $retorno = $response->getBody();
                $retorno = json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function callbackCertsing($filedata, $nrorg) {
        $this->iniciarDependencias($nrorg);
        $assinouOuRecusou = null;
        
        try {
            $action = $filedata['action'];
            
            if (($action == 'SIGNATURE-ELETRONIC') || ($action == 'SIGNATURE-DIGITAL')) {
                $assinouOuRecusou = 'A';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];

                $identifier = str_replace(".", "", $identifier);
                $identifier = str_replace("-", "", $identifier);
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId,
                    "CPFSIGNATORY" => $identifier
                );
                $getNrCallSignature = $this->assinaturaFactory->getCallSignature($params);
                
                if (count($getNrCallSignature) > 0) {

                    $seqCallSign = $getNrCallSignature[0]['NRCALLSIGNATURE'];
                    $nrcall = $getNrCallSignature[0]['NRCALL'];
                
                    $params = array(
                        "IDSITUACAO"        => 'A',
                        "DSJUSTIFICARECUSA" => null,
                        "NRCALLSIGNATURE"  => $seqCallSign,
                        "CPFSIGNATORY" => $identifier
                    );
                    
                    $this->assinaturaFactory->updateCallSignature($params);
                }

                $this->posCallback($documentId, $identifier);
            }
            else if ($action == 'SIGNATURE-NOK') {
                $assinouOuRecusou = 'R';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                $message    = $filedata['message'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId,
                    "CPFSIGNATORY" => $identifier
                );
                
                $getNrCallSignature = $this->assinaturaFactory->getCallSignature($params);
                
                if (count($getNrCallSignature) > 0) {

                    $seqCallSign = $getNrCallSignature[0]['NRCALLSIGNATURE'];
                    $nrcall = $getNrCallSignature[0]['NRCALL'];
                    
                    $params = array(
                        "IDSITUACAO"        => 'R',
                        "DSJUSTIFICARECUSA" => $message,
                        "NRCALLSIGNATURE"   => $seqCallSign,
                        "CPFSIGNATORY"      => $identifier
                    );
                    
                    $this->assinaturaFactory->updateCallSignature($params);
                    
                }
            }
        }
        catch(\Exception $e) {
            //file_put_contents("/home/developer/workfolder/TESTE/erro.txt", $e->getMessage());
            
            $dserro = "Erro ao atualizar a situação do documento: " . $e->getMessage();
            
            $nrlogassinatdigital = $this->newCode->getCodeByTableName("CP_LOGCALLSIGNATURE", $nrorg, 1, 10, true);
            
            $params = array(
                "NRLOGCALLSIGNATURE"  => $nrlogassinatdigital,
                "NRCALL"              => isset($nrcall) ? $nrcall : null,
                "IDDOCUMCERTSIGN"     => isset($documentId) ? $documentId : null,
                "CPFSIGNATORY"        => isset($identifier) ? $identifier : null,
                "DSERRO"              => $dserro,
                "NRORG"               => $nrorg
            );
            
            $this->assinaturaFactory->insertLogCallSignature($params);
            
            throw $e;
        }
    }

    public function posCallback($documentId, $identifier) {
        try {
           
            $params = array(
                "IDDOCUMCERTSIGN" => $documentId,
                "CPFSIGNATORY"   => $identifier
            );

            $getCallSignature = $this->assinaturaFactory->getCallSignature($params);  
            
            if (count($getCallSignature) > 0){

                $nrcall = $getCallSignature[0]["NRCALL"];
                $comment = $getCallSignature[0]["CALL_COMMENT"];
                $callRelationship = $getCallSignature[0]["CALL_RELATIONSHIP"];
                $reference = $getCallSignature[0]["CALL_REFERENCE"];
                $product = $getCallSignature[0]["CALL_PRODUCT"];
                $classification = $getCallSignature[0]["CALL_CLASSIFICATION"];
                $board = $getCallSignature[0]["CALL_BOARD"]; 
                $description = $getCallSignature[0]["CALL_DESCRIPTION"];
                
                \Util\Instance\Service::getCallStatic()->approveIssue($nrcall, $comment, $callRelationship, $reference, $product, $classification, $board, $description);
            
            }
            
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function incluirSignatario($documentId, $signers, $eletrSigners) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantAdd', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $body = $this->buildJsontoIncSignatario($documentId, $signers, $eletrSigners);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
             
            // Request body
            $request->setBody($body);
        
            try {
                $response = $request->send();
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buildJsontoIncSignatario($documentId, $signers, $eletrSigners){

        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "documentId" => $documentId,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );
        
        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }
    
    public function excluirSignatario($documentId, $idsignatcertsign, $nrcpfsignatario) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantDiscard', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documentId" => $documentId,
                "individualIdentificationCode" => $nrcpfsignatario
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function reenviarSolicitacao($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/SendReminder', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documents" => [$documentId]
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
            
            try {
                $response = $request->send();

                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function excluirdocumento($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/delete/'.$documentId, \Http_Request2::METHOD_DELETE);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_DELETE);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
            try {
                $response = $request->send();
                
                $retorno = $response->getBody();
                $retorno = json_decode($retorno, true);
                
                $return['response'] = $retorno;
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function dowloadDocumento($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/package?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave,
                'includeOriginal' => 'True',
                'includeManifest' => 'True',
                'zipped' => 'True'
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getTokenCertsign() {

        try {
           
           $nrorg = $this->environment->getNrOrg();
               
           $params = array(
                "NRORG" => $nrorg
           );
            
           $token = $this->assinaturaFactory->getTokenCertsign($params);
        
           $token = $token[0]["DSTOKENCERTSIGN"];   
           
           return $token;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
    }

    public function verificarAssinatura($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/ValidateSignatures?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave 
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);

                $return['response'] = $retorno;
                
                return $return['response'];
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
}